// Kinetophone_builder_model.hpp
//
// Copyright 2012-2013 Roan Trail, Inc.
//
// This file is part of Kinetophone.
//
// Kinetophone is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 2 of the License,
// or (at your option) any later version.
//
// Kinetophone is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.  You should have received
// a copy of the GNU General Public License along with Kinetophone. If
// not, see <http://www.gnu.org/licenses/>.

#ifndef ROAN_TRAIL_KINETOPHONE_APP_KINETOPHONE_BUILDER_MODEL_HPP_
#define ROAN_TRAIL_KINETOPHONE_APP_KINETOPHONE_BUILDER_MODEL_HPP_

#include <kinetophone/Movie_builder_model.hpp>
#include <string>

namespace Gdk
{
  class Pixbuf;
}

namespace Glib
{
  template <class T> class RefPtr;
}

namespace Roan_trail
{
  namespace Kinetophone
  {
    class Error_param;
    class Kinetophone_builder_config;
    template <class Image> class Slide_collection;
  }

  namespace Kinetophone_app
  {
    class Kinetophone_builder_config;

    class Kinetophone_builder_model : public Roan_trail::Kinetophone::Movie_builder_model
    {
    public:
      // enum
      enum Slides_source_type
        {
          directory_source,
          PDF_source
        };
      // constructor/destructor
      Kinetophone_builder_model(const Kinetophone_builder_config& builder_config);
      ~Kinetophone_builder_model();
      // accessors
      const Roan_trail::Kinetophone::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* slides() const
      { return m_slides; }
      // load
      bool load_from_XML(const std::string& file_path, Roan_trail::Kinetophone::Error_param& return_error);
    protected:
      // invariant check
      bool mf_invariant(bool check_base_class = true) const;
    private:
      const Kinetophone_builder_config& m_builder_config;
      Roan_trail::Kinetophone::Slide_collection<Glib::RefPtr<Gdk::Pixbuf> >* m_slides;
      Slides_source_type m_slides_source;
      // prevent compiler from generating
      Kinetophone_builder_model(const Kinetophone_builder_model& builder_model);
      Kinetophone_builder_model& operator=(const Kinetophone_builder_model& builder_model);
    };
  }
}

#endif // ROAN_TRAIL_KINETOPHONE_APP_KINETOPHONE_BUILDER_MODEL_HPP_
